@@ -172,16 +172,16 @@ ldsm_analyze_path (const gchar *path)
 
 static gboolean
 ldsm_notify_for_mount (LdsmMountInfo *mount,
-                       gboolean       has_disk_analyzer,
                        gboolean       multiple_volumes,
                        gboolean       other_usable_volumes)
 {
-        gchar  *name;
+        gchar  *name, *program;
         gint64 free_space;
         gint response;
         gboolean has_trash;
+        gboolean has_disk_analyzer;
         gboolean retval = TRUE;
-        const gchar *path;
+        gchar *path;
 
         /* Don't show a dialog if one is already displayed */
         if (dialog)
@@ -190,7 +190,11 @@ ldsm_notify_for_mount (LdsmMountInfo *mount,
         name = g_unix_mount_guess_name (mount->mount);
         free_space = (gint64) mount->buf.f_frsize * (gint64) mount->buf.f_bavail;
         has_trash = ldsm_mount_has_trash (mount);
-        path = g_unix_mount_get_mount_path (mount->mount);
+        path = g_strdup (g_unix_mount_get_mount_path (mount->mount));
+
+        program = g_find_program_in_path (DISK_SPACE_ANALYZER);
+        has_disk_analyzer = (program != NULL);
+        g_free (program);
 
         dialog = gsd_ldsm_dialog_new (other_usable_volumes,
                                       multiple_volumes,
@@ -214,7 +218,7 @@ ldsm_notify_for_mount (LdsmMountInfo *mount,
                 break;
         case GSD_LDSM_DIALOG_RESPONSE_ANALYZE:
                 retval = FALSE;
-                ldsm_analyze_path (g_unix_mount_get_mount_path (mount->mount));
+                ldsm_analyze_path (path);
                 break;
         case GSD_LDSM_DIALOG_RESPONSE_EMPTY_TRASH:
                 retval = TRUE;
@@ -228,6 +232,8 @@ ldsm_notify_for_mount (LdsmMountInfo *mount,
                 g_assert_not_reached ();
         }
 
+        g_free (path);
+
         return retval;
 }
 
@@ -315,6 +321,7 @@ ldsm_mount_should_ignore (GUnixMountEntry *mount)
                 "procfs",
                 "ptyfs",
                 "selinuxfs",
+                "linsysfs",
                 "sysfs",
                 "tmpfs",
                 "usbfs",
@@ -358,7 +365,6 @@ ldsm_free_mount_info (gpointer data)
 
 static void
 ldsm_maybe_warn_mounts (GList *mounts,
-                        gboolean has_disk_analyzer,
                         gboolean multiple_volumes,
                         gboolean other_usable_volumes)
 {
@@ -417,7 +423,7 @@ ldsm_maybe_warn_mounts (GList *mounts,
                 }
 
                 if (show_notify) {
-                        if (ldsm_notify_for_mount (mount_info, has_disk_analyzer, multiple_volumes, other_usable_volumes))
+                        if (ldsm_notify_for_mount (mount_info, multiple_volumes, other_usable_volumes))
                                 done = TRUE;
                 }
         }
@@ -430,17 +436,11 @@ ldsm_check_all_mounts (gpointer data)
         GList *l;
         GList *check_mounts = NULL;
         GList *full_mounts = NULL;
-        char *program;
-        gboolean has_disk_analyzer;
         guint number_of_mounts;
         guint number_of_full_mounts;
         gboolean multiple_volumes = FALSE;
         gboolean other_usable_volumes = FALSE;
 
-        program = g_find_program_in_path (DISK_SPACE_ANALYZER);
-        has_disk_analyzer = (program != NULL);
-        g_free (program);
-
         /* We iterate through the static mounts in /etc/fstab first, seeing if
          * they're mounted by checking if the GUnixMountPoint has a corresponding GUnixMountEntry.
          * Iterating through the static mounts means we automatically ignore dynamically mounted media.
@@ -508,7 +508,7 @@ ldsm_check_all_mounts (gpointer data)
         if (number_of_mounts > number_of_full_mounts)
                 other_usable_volumes = TRUE;
 
-        ldsm_maybe_warn_mounts (full_mounts, has_disk_analyzer, multiple_volumes,
+        ldsm_maybe_warn_mounts (full_mounts, multiple_volumes,
                                 other_usable_volumes);
 
         g_list_free (check_mounts);
